import 'package:shared_preferences/shared_preferences.dart';
import 'dart:convert';
import 'package:http/http.dart' as http;
import 'misc.dart';

// data structure for storing reward data
class RewardData {
  // includes an array of json objects sorted in decending point order with the following fields:
  // name, points
  List<Map<String, dynamic>> rewardData = [];
  // include a date that the data was last updated
  DateTime lastUpdated = DateTime.now();

  // constructor
  RewardData(this.rewardData, this.lastUpdated);

  // constructor to create an empty rewardData object
  RewardData.empty();

  // static method to get the reward data from local storage or the backend if it is not in local storage or if it is more than 1 hour old
  static Future<RewardData> getRewardData(
      [bool invalidateCache = false]) async {
    RewardData? rewardData = RewardData.empty();

    // if the cache should be invalidated, get the reward data from the backend
    if (invalidateCache) {
      rewardData = await getRewardDataFromBackend();
      // save the reward data to local storage
      await saveRewardDataToLocalStorage(rewardData);
      return rewardData;
    }
    // get the reward data from local storage
    rewardData = await getRewardDataFromLocalStorage();
    // if the reward data is null or if it is more than 1 hour old, get the reward data from the backend
    if (rewardData == null ||
        DateTime.now().difference(rewardData.lastUpdated).inMinutes >= 1) {
      rewardData = await getRewardDataFromBackend();
      // save the reward data to local storage
      await saveRewardDataToLocalStorage(rewardData);
    }

    // print(
    // "rewardData: ${rewardData.rewardData}\nlastUpdated: ${rewardData.lastUpdated}");
    return rewardData;
  }

  Map<String, dynamic> toJson() {
    return {
      'rewardData': rewardData,
      'lastUpdated': lastUpdated.toIso8601String(),
    };
  }

  // static method to get the reward data from local storage
  static Future<RewardData?> getRewardDataFromLocalStorage() async {
    // get the shared preferences
    SharedPreferences prefs = await SharedPreferences.getInstance();
    // get the reward data from local storage
    var rewardDataJson = prefs.getString('rewardData');
    // if the reward data is not null, convert it to a rewardData object
    if (rewardDataJson != null) {
      return RewardData.fromLocalArrayofJson(rewardDataJson);
    }
    return null;
  }

  // static method to get the reward data from the backend
  static Future<RewardData> getRewardDataFromBackend() async {
    // get the reward data from the backend
    var response = await http.post(Uri.parse('$backendUrl/shop/items'));
    // if the response is successful, convert the response body to a rewardData object
    if (response.statusCode == 200) {
      return RewardData.fromArrayofJson(response.body);
    }
    // if the response is not successful, throw an error
    else {
      throw Exception('Failed to get reward data from backend');
    }
  }

  static Future<void> saveRewardDataToLocalStorage(
      RewardData rewardData) async {
    // get the shared preferences
    SharedPreferences prefs = await SharedPreferences.getInstance();
    // save the reward data to local storage
    prefs.setString('rewardData', jsonEncode(rewardData.toJson()));
  }

  static RewardData fromArrayofJson(String jsonArr) {
    // Decode the JSON array into a dynamic object
    print("jsonArr: $jsonArr");
    List<dynamic> jsonList = jsonDecode(jsonArr);
    // Convert the dynamic object to a List<Map<String, dynamic>>
    List<Map<String, dynamic>> jsonArray =
        jsonList.map((dynamic item) => item as Map<String, dynamic>).toList();

    // Check that the decoded object is a List
    // Map each element of the list to a Map<String, dynamic> object
    return RewardData(jsonArray, DateTime.now());
  }

  static RewardData fromLocalArrayofJson(String jsonArr) {
    // Decode the JSON array into a dynamic object
    Map<String, dynamic> jsonMap = jsonDecode(jsonArr);
    List<dynamic> jsonList = jsonMap['rewardData'];
    DateTime lastUpdated = DateTime.parse(jsonMap['lastUpdated']);
    return RewardData(
        jsonList.map((dynamic item) => item as Map<String, dynamic>).toList(),
        lastUpdated);
  }
}
